@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009, Nokia
+ * Copyright (C) 2009, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -34,6 +34,10 @@
 
 #include <glib/gstdio.h>
 
+#ifndef O_LARGEFILE
+# define O_LARGEFILE 0
+#endif
+
 #include <libtracker-common/tracker-crc32.h>
 
 #include "tracker-db-journal.h"
@@ -57,6 +61,11 @@ typedef enum {
 	DATA_FORMAT_GRAPH            = 1 << 3
 } DataFormat;
 
+typedef enum {
+	TRANSACTION_FORMAT_DATA      = 1 << 0,
+	TRANSACTION_FORMAT_ONTOLOGY  = 1 << 1,
+} TransactionFormat;
+
 static struct {
 	gchar *filename;
 	GMappedFile *file;
@@ -237,7 +246,7 @@ tracker_db_journal_init (const gchar *filename, gboolean truncate)
 	g_free (directory);
 
 	mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP;
-	flags = O_WRONLY | O_APPEND | O_CREAT;
+	flags = O_WRONLY | O_APPEND | O_CREAT | O_LARGEFILE;
 	if (truncate) {
 		/* existing journal contents are invalid: reindex where journal
 		 * does not even contain a single valid entry
@@ -276,7 +285,7 @@ tracker_db_journal_init (const gchar *filename, gboolean truncate)
 		writer.cur_block[4] = 'g';
 		writer.cur_block[5] = '\0';
 		writer.cur_block[6] = '0';
-		writer.cur_block[7] = '1';
+		writer.cur_block[7] = '2';
 
 		if (!write_all_data (writer.journal, writer.cur_block, 8)) {
 			g_free (writer.journal_filename);
@@ -352,6 +361,44 @@ tracker_db_journal_start_transaction (time_t time)
 	cur_setnum (writer.cur_block, &writer.cur_pos, time);
 	writer.cur_block_len += sizeof (gint32);
 
+	/* Add format */
+	cur_block_maybe_expand (sizeof (gint32));
+	cur_setnum (writer.cur_block, &writer.cur_pos, TRANSACTION_FORMAT_DATA);
+	writer.cur_block_len += sizeof (gint32);
+
+	return TRUE;
+}
+
+
+gboolean
+tracker_db_journal_start_ontology_transaction (time_t time)
+{
+	guint size;
+
+	g_return_val_if_fail (writer.journal > 0, FALSE);
+
+	size = sizeof (guint32) * 3;
+	cur_block_maybe_expand (size);
+
+	/* Leave space for size, amount and crc
+	 * Check and keep in sync the offset variable at
+	 * tracker_db_journal_commit_db_transaction too */
+
+	memset (writer.cur_block, 0, size);
+
+	writer.cur_pos = writer.cur_block_len = size;
+	writer.cur_entry_amount = 0;
+
+	/* add timestamp */
+	cur_block_maybe_expand (sizeof (gint32));
+	cur_setnum (writer.cur_block, &writer.cur_pos, time);
+	writer.cur_block_len += sizeof (gint32);
+
+	/* Add format */
+	cur_block_maybe_expand (sizeof (gint32));
+	cur_setnum (writer.cur_block, &writer.cur_pos, TRANSACTION_FORMAT_ONTOLOGY);
+	writer.cur_block_len += sizeof (gint32);
+
 	return TRUE;
 }
 
@@ -666,7 +713,7 @@ tracker_db_journal_reader_init (const gchar *filename)
 		return FALSE;
 	}
 
-	if (memcmp (reader.current, "trlog\00001", 8)) {
+	if (memcmp (reader.current, "trlog\00002", 8)) {
 		tracker_db_journal_reader_shutdown ();
 		return FALSE;
 	}
@@ -787,6 +834,7 @@ tracker_db_journal_reader_next (GError **error)
 		guint32 entry_size_check;
 		guint32 crc;
 		guint32 crc_check;
+		TransactionFormat t_kind;
 
 		/* Check the end is not before where we currently are */
 		if (reader.current >= reader.end) {
@@ -872,7 +920,14 @@ tracker_db_journal_reader_next (GError **error)
 		reader.time = read_uint32 (reader.current);
 		reader.current += 4;
 
-		reader.type = TRACKER_DB_JOURNAL_START_TRANSACTION;
+		t_kind = read_uint32 (reader.current);
+		reader.current += 4;
+
+		if (t_kind == TRANSACTION_FORMAT_DATA)
+			reader.type = TRACKER_DB_JOURNAL_START_TRANSACTION;
+		else
+			reader.type = TRACKER_DB_JOURNAL_START_ONTOLOGY_TRANSACTION;
+
 		return TRUE;
 	} else if (reader.amount_of_triples == 0) {
 		/* end of transaction */
@@ -1093,3 +1148,11 @@ tracker_db_journal_reader_get_statement_id (gint *g_id,
 
 	return TRUE;
 }
+
+gdouble
+tracker_db_journal_reader_get_progress (void)
+{
+	gdouble percent = ((gdouble)(reader.end - reader.start));
+	return ((gdouble)(reader.current - reader.start)) / percent;
+}
+
