@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2006, Laurent Aguerreche (laurent.aguerreche@free.fr)
- * Copyright (C) 2007, Mr Jamie McCracken (jamiemcc@gnome.org)
- * Copyright (C) 2008, Nokia
+ * Copyright (C) 2006, Laurent Aguerreche <laurent.aguerreche@free.fr>
+ * Copyright (C) 2007, Jamie McCracken <jamiemcc@gnome.org>
+ * Copyright (C) 2008, Nokia <ivan.frade@nokia.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -35,10 +35,7 @@ long long int llroundl(long double x);
 #include <gst/gst.h>
 #include <gst/tag/tag.h>
 
-#include <libtracker-common/tracker-type-utils.h>
-#include <libtracker-common/tracker-file-utils.h>
-#include <libtracker-common/tracker-ontology.h>
-#include <libtracker-common/tracker-utils.h>
+#include <libtracker-client/tracker.h>
 
 #include <libtracker-extract/tracker-extract.h>
 
@@ -53,14 +50,6 @@ long long int llroundl(long double x);
 #define GST_TAG_CLASSIFICATION "classification"
 #endif
 
-#define NMM_PREFIX TRACKER_NMM_PREFIX
-#define NFO_PREFIX TRACKER_NFO_PREFIX
-#define NIE_PREFIX TRACKER_NIE_PREFIX
-#define DC_PREFIX TRACKER_DC_PREFIX
-#define NCO_PREFIX TRACKER_NCO_PREFIX
-#define RDF_PREFIX TRACKER_RDF_PREFIX
-#define RDF_TYPE RDF_PREFIX "type"
-
 /* Some additional tagreadbin tags (FIXME until they are defined upstream)*/
 #ifndef GST_TAG_CHANNEL
 #define GST_TAG_CHANNEL "channels"
@@ -90,7 +79,7 @@ typedef enum {
 	EXTRACT_MIME_AUDIO,
 	EXTRACT_MIME_VIDEO,
 	EXTRACT_MIME_IMAGE,
-	EXTRACT_MIME_3GPP,
+	EXTRACT_MIME_GUESS,
 } ExtractMime;
 
 typedef struct {
@@ -113,6 +102,7 @@ typedef struct {
 	gint            video_fps_d;
 	gint            audio_channels;
 	gint            audio_samplerate;
+	gint            aspect_ratio;
 
 	/* Tags and data */
 	GstTagList     *tagcache;
@@ -134,7 +124,7 @@ static void extract_gstreamer_video (const gchar          *uri,
 static void extract_gstreamer_image (const gchar          *uri,
                                      TrackerSparqlBuilder *preupdate,
                                      TrackerSparqlBuilder *metadata);
-static void extract_gstreamer_3gpp  (const gchar          *uri,
+static void extract_gstreamer_guess (const gchar          *uri,
                                      TrackerSparqlBuilder *preupdate,
                                      TrackerSparqlBuilder *metadata);
 
@@ -142,8 +132,9 @@ static TrackerExtractData data[] = {
 	{ "audio/*", extract_gstreamer_audio },
 	{ "video/*", extract_gstreamer_video },
 	{ "image/*", extract_gstreamer_image },
-	/* mime type guessing returns video/3gpp also for 3gpp audio files */
-	{ "video/3gpp", extract_gstreamer_3gpp },
+	/* mime type guessing returns for instance video/3gpp also for 3gpp audio files */
+	{ "video/3gpp", extract_gstreamer_guess },
+	{ "video/mp4", extract_gstreamer_guess },
 	{ NULL, NULL }
 };
 
@@ -264,12 +255,14 @@ add_fraction_gst_tag (TrackerSparqlBuilder         *metadata,
 
 	ret = gst_tag_list_copy_value (&n, tag_list, tag);
 
-	f = (gfloat)gst_value_get_fraction_numerator (&n)/
+	if (ret) {
+		f = (gfloat)gst_value_get_fraction_numerator (&n)/
 		gst_value_get_fraction_denominator (&n);
 
-	if (ret) {
 		tracker_sparql_builder_predicate (metadata, key);
 		tracker_sparql_builder_object_double (metadata, (gdouble) f);
+
+		g_value_unset (&n);
 	}
 }
 
@@ -286,6 +279,16 @@ add_y_date_gst_tag (TrackerSparqlBuilder  *metadata,
 	date = NULL;
 	ret = gst_tag_list_get_date (tag_list, tag, &date);
 
+	if (ret) {
+		if (date && g_date_valid (date)) {
+			if (date->julian)
+				ret = g_date_valid_julian (date->julian_days);
+			if (date->dmy)
+				ret = g_date_valid_dmy (date->day, date->month, date->year);
+		} else
+			ret = FALSE;
+	}
+
 	if (ret) {
 		gchar buf[25];
 
@@ -395,9 +398,10 @@ extract_stream_metadata_tagreadbin (MetadataExtractor *extractor,
 		add_int_gst_tag (metadata, uri, "nfo:channels", extractor->tagcache, GST_TAG_CHANNEL);
 		add_int_gst_tag (metadata, uri, "nfo:sampleRate", extractor->tagcache, GST_TAG_RATE);
 		add_time_gst_tag (metadata, uri, "nfo:duration", extractor->tagcache, GST_TAG_DURATION);
-		if (extractor->mime == EXTRACT_MIME_AUDIO) {
-			add_time_gst_tag (metadata, uri, "nmm:length", extractor->tagcache, GST_TAG_DURATION);
-		}
+	}
+
+	if (extractor->mime == EXTRACT_MIME_IMAGE || extractor->mime == EXTRACT_MIME_VIDEO) {
+		add_fraction_gst_tag (metadata, uri, "nfo:aspectRatio", extractor->tagcache, GST_TAG_PIXEL_RATIO);
 	}
 
 	add_int_gst_tag (metadata, uri, "nfo:height", extractor->tagcache, GST_TAG_HEIGHT);
@@ -414,6 +418,7 @@ extract_stream_metadata_decodebin (MetadataExtractor *extractor,
                                    TrackerSparqlBuilder         *metadata)
 {
 	if (extractor->mime != EXTRACT_MIME_IMAGE) {
+
 		if (extractor->audio_channels >= 0) {
 			add_uint_info (metadata, uri, "nfo:channels", extractor->audio_channels);
 		}
@@ -424,9 +429,10 @@ extract_stream_metadata_decodebin (MetadataExtractor *extractor,
 
 		if (extractor->duration >= 0) {
 			add_int64_info (metadata, uri, "nfo:duration", extractor->duration);
-			if (extractor->mime == EXTRACT_MIME_AUDIO) {
-				add_int64_info (metadata, uri, "nmm:length", extractor->duration);
-			}
+		}
+	} else {
+		if (extractor->aspect_ratio >= 0) {
+			add_uint_info (metadata, uri, "nfo:aspectRatio", extractor->aspect_ratio);
 		}
 	}
 
@@ -453,6 +459,7 @@ extract_metadata (MetadataExtractor      *extractor,
                   const gchar            *uri,
                   TrackerSparqlBuilder   *preupdate,
                   TrackerSparqlBuilder   *metadata,
+                  gchar                 **artist,
                   gchar                 **album,
                   gchar                 **scount)
 {
@@ -469,7 +476,7 @@ extract_metadata (MetadataExtractor      *extractor,
 		gchar *composer_uri = NULL;
 		gchar *album_uri = NULL;
 
-		if (extractor->mime == EXTRACT_MIME_3GPP) {
+		if (extractor->mime == EXTRACT_MIME_GUESS) {
 			gchar *video_codec = NULL, *audio_codec = NULL;
 
 			gst_tag_list_get_string (extractor->tagcache, GST_TAG_VIDEO_CODEC, &video_codec);
@@ -488,12 +495,12 @@ extract_metadata (MetadataExtractor      *extractor,
 
 		/* General */
 		if (extractor->mime == EXTRACT_MIME_AUDIO || extractor->mime == EXTRACT_MIME_VIDEO) {
-			gchar *performer = NULL, *artist = NULL;
+			gchar *performer = NULL, *artist_local = NULL;
 
 			gst_tag_list_get_string (extractor->tagcache, GST_TAG_PERFORMER, &performer);
-			gst_tag_list_get_string (extractor->tagcache, GST_TAG_ARTIST, &artist);
+			gst_tag_list_get_string (extractor->tagcache, GST_TAG_ARTIST, &artist_local);
 
-			s = tracker_coalesce (2, performer, artist);
+			s = tracker_coalesce (2, performer, artist_local);
 
 			if (s) {
 				performer_uri = tracker_uri_printf_escaped ("urn:artist:%s", s);
@@ -506,7 +513,7 @@ extract_metadata (MetadataExtractor      *extractor,
 				tracker_sparql_builder_object_unvalidated (preupdate, s);
 				tracker_sparql_builder_insert_close (preupdate);
 
-				g_free (s);
+				*artist = s;
 			}
 
 			s = NULL;
@@ -537,6 +544,7 @@ extract_metadata (MetadataExtractor      *extractor,
 		if (s) {
 			gboolean has_it;
 			guint count;
+			gdouble gain;
 
 			needs_audio = TRUE;
 
@@ -552,8 +560,8 @@ extract_metadata (MetadataExtractor      *extractor,
 
 			tracker_sparql_builder_insert_close (preupdate);
 
-			has_it = gst_tag_list_get_uint (extractor->tagcache, 
-			                                GST_TAG_TRACK_COUNT, 
+			has_it = gst_tag_list_get_uint (extractor->tagcache,
+			                                GST_TAG_TRACK_COUNT,
 			                                &count);
 
 			if (has_it) {
@@ -578,8 +586,8 @@ extract_metadata (MetadataExtractor      *extractor,
 				tracker_sparql_builder_insert_close (preupdate);
 			}
 
-			has_it = gst_tag_list_get_uint (extractor->tagcache, 
-			                                GST_TAG_ALBUM_VOLUME_NUMBER, 
+			has_it = gst_tag_list_get_uint (extractor->tagcache,
+			                                GST_TAG_ALBUM_VOLUME_NUMBER,
 			                                &count);
 
 			if (has_it) {
@@ -604,7 +612,60 @@ extract_metadata (MetadataExtractor      *extractor,
 				tracker_sparql_builder_insert_close (preupdate);
 			}
 
+			has_it = gst_tag_list_get_double (extractor->tagcache,
+			                                  GST_TAG_ALBUM_GAIN,
+			                                  &gain);
+
+			if (has_it) {
+				tracker_sparql_builder_delete_open (preupdate, NULL);
+				tracker_sparql_builder_subject_iri (preupdate, album_uri);
+				tracker_sparql_builder_predicate (preupdate, "nmm:albumGain");
+				tracker_sparql_builder_object_variable (preupdate, "unknown");
+				tracker_sparql_builder_delete_close (preupdate);
+
+				tracker_sparql_builder_where_open (preupdate);
+				tracker_sparql_builder_subject_iri (preupdate, album_uri);
+				tracker_sparql_builder_predicate (preupdate, "nmm:albumGain");
+				tracker_sparql_builder_object_variable (preupdate, "unknown");
+				tracker_sparql_builder_where_close (preupdate);
+
+				tracker_sparql_builder_insert_open (preupdate, NULL);
+
+				tracker_sparql_builder_subject_iri (preupdate, album_uri);
+				tracker_sparql_builder_predicate (preupdate, "nmm:albumGain");
+				tracker_sparql_builder_object_double (preupdate, gain);
+
+				tracker_sparql_builder_insert_close (preupdate);
+			}
+
+			has_it = gst_tag_list_get_double (extractor->tagcache,
+			                                  GST_TAG_ALBUM_PEAK,
+			                                  &gain);
+
+			if (has_it) {
+				tracker_sparql_builder_delete_open (preupdate, NULL);
+				tracker_sparql_builder_subject_iri (preupdate, album_uri);
+				tracker_sparql_builder_predicate (preupdate, "nmm:albumPeakGain");
+				tracker_sparql_builder_object_variable (preupdate, "unknown");
+				tracker_sparql_builder_delete_close (preupdate);
+
+				tracker_sparql_builder_where_open (preupdate);
+				tracker_sparql_builder_subject_iri (preupdate, album_uri);
+				tracker_sparql_builder_predicate (preupdate, "nmm:albumPeakGain");
+				tracker_sparql_builder_object_variable (preupdate, "unknown");
+				tracker_sparql_builder_where_close (preupdate);
+
+				tracker_sparql_builder_insert_open (preupdate, NULL);
+
+				tracker_sparql_builder_subject_iri (preupdate, album_uri);
+				tracker_sparql_builder_predicate (preupdate, "nmm:albumPeakGain");
+				tracker_sparql_builder_object_double (preupdate, gain);
+
+				tracker_sparql_builder_insert_close (preupdate);
+			}
+
 			*album = s;
+
 		}
 
 		if (extractor->mime == EXTRACT_MIME_AUDIO)
@@ -615,7 +676,7 @@ extract_metadata (MetadataExtractor      *extractor,
 		if (needs_audio) {
 			tracker_sparql_builder_object (metadata, "nmm:MusicPiece");
 			tracker_sparql_builder_object (metadata, "nfo:Audio");
-		} 
+		}
 
 		if (extractor->mime == EXTRACT_MIME_VIDEO) {
 			tracker_sparql_builder_object (metadata, "nmm:Video");
@@ -668,8 +729,6 @@ extract_metadata (MetadataExtractor      *extractor,
 
 			add_double_gst_tag (metadata, uri, "nfo:gain", extractor->tagcache, GST_TAG_TRACK_GAIN);
 			add_double_gst_tag (metadata, uri, "nfo:peakGain", extractor->tagcache, GST_TAG_TRACK_PEAK);
-			add_double_gst_tag (metadata, uri, "nfo:albumGain", extractor->tagcache, GST_TAG_ALBUM_GAIN);
-			add_double_gst_tag (metadata, uri, "nfo:albumPeakGain", extractor->tagcache, GST_TAG_ALBUM_PEAK);
 
 			if (performer_uri) {
 				tracker_sparql_builder_predicate (metadata, "nmm:performer");
@@ -692,8 +751,25 @@ extract_metadata (MetadataExtractor      *extractor,
 		g_free (album_uri);
 
 		add_string_gst_tag (metadata, uri, "nfo:codec", extractor->tagcache, GST_TAG_AUDIO_CODEC);
+	} else {
+		if (extractor->mime == EXTRACT_MIME_AUDIO)
+			needs_audio = TRUE;
+
+		tracker_sparql_builder_predicate (metadata, "a");
+
+		if (needs_audio) {
+			tracker_sparql_builder_object (metadata, "nmm:MusicPiece");
+			tracker_sparql_builder_object (metadata, "nfo:Audio");
+		}
+
+		if (extractor->mime == EXTRACT_MIME_VIDEO) {
+			tracker_sparql_builder_object (metadata, "nmm:Video");
+		} else if (!needs_audio) {
+			tracker_sparql_builder_object (metadata, "nfo:Image");
+		}
 	}
 
+
 	if (use_tagreadbin) {
 		extract_stream_metadata_tagreadbin (extractor, uri, metadata);
 	} else {
@@ -798,7 +874,8 @@ add_stream_tag (void *obj, void *data_)
 				                                     &extractor->video_fps_n,
 				                                     &extractor->video_fps_d) ) &&
 				        (gst_structure_get_int (s, "width", &extractor->video_width)) &&
-				        (gst_structure_get_int (s, "height", &extractor->video_height)))) {
+				        (gst_structure_get_int (s, "height", &extractor->video_height)) &&
+				        (gst_structure_get_int (s, "pixel-aspect-ratio", &extractor->aspect_ratio)))) {
 					return;
 				}
 			}
@@ -1000,7 +1077,7 @@ tracker_extract_gstreamer (const gchar *uri,
                            ExtractMime  type)
 {
 	MetadataExtractor *extractor;
-	gchar *album, *scount;
+	gchar *artist, *album, *scount;
 
 	g_return_if_fail (uri);
 	g_return_if_fail (metadata);
@@ -1029,6 +1106,7 @@ tracker_extract_gstreamer (const gchar *uri,
 	extractor->video_fps_n = extractor->video_fps_d = -1;
 	extractor->video_height = extractor->video_width = -1;
 	extractor->audio_channels = -1;
+	extractor->aspect_ratio = -1;
 	extractor->audio_samplerate = -1;
 
 	if (use_tagreadbin) {
@@ -1065,18 +1143,20 @@ tracker_extract_gstreamer (const gchar *uri,
 
 	album = NULL;
 	scount = NULL;
+	artist = NULL;
 
-	extract_metadata (extractor, uri, preupdate, metadata, &album, &scount);
+	extract_metadata (extractor, uri, preupdate, metadata, &artist, &album, &scount);
 
 	tracker_albumart_process (extractor->album_art_data,
 	                          extractor->album_art_size,
 	                          extractor->album_art_mime,
-	                          /* g_hash_table_lookup (metadata, "Audio:Artist") */ NULL,
+	                          artist,
 	                          album,
 	                          uri);
 
 	g_free (scount);
 	g_free (album);
+	g_free (artist);
 
 	gst_element_set_state (extractor->pipeline, GST_STATE_NULL);
 	gst_element_get_state (extractor->pipeline, NULL, NULL, TRACKER_EXTRACT_GUARD_TIMEOUT* GST_SECOND);
@@ -1116,11 +1196,11 @@ extract_gstreamer_image (const gchar          *uri,
 }
 
 static void
-extract_gstreamer_3gpp (const gchar          *uri,
+extract_gstreamer_guess (const gchar          *uri,
                         TrackerSparqlBuilder *preupdate,
 			TrackerSparqlBuilder *metadata)
 {
-	tracker_extract_gstreamer (uri, preupdate, metadata, EXTRACT_MIME_3GPP);
+	tracker_extract_gstreamer (uri, preupdate, metadata, EXTRACT_MIME_GUESS);
 }
 
 TrackerExtractData *
