@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2006, Mr Jamie McCracken (jamiemcc@gnome.org)
- * Copyright (C) 2008, Nokia
+ * Copyright (C) 2006, Jamie McCracken <jamiemcc@gnome.org>
+ * Copyright (C) 2008, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -27,7 +27,7 @@
 #include <libtracker-common/tracker-log.h>
 #include <libtracker-common/tracker-utils.h>
 #include <libtracker-common/tracker-type-utils.h>
-#include <libtracker-common/tracker-ontology.h>
+#include <libtracker-common/tracker-ontologies.h>
 
 #include <libtracker-db/tracker-db-dbus.h>
 
@@ -66,6 +66,8 @@ typedef struct {
 } TrackerDBusMethodInfo;
 
 
+static void tracker_resources_finalize (GObject *object);
+
 static guint signals[LAST_SIGNAL] = { 0 };
 
 static void
@@ -80,18 +82,6 @@ free_event_sources (TrackerResourcesPrivate *priv)
 	}
 }
 
-static void
-tracker_resources_finalize (GObject      *object)
-{
-	TrackerResourcesPrivate *priv;
-
-	priv = TRACKER_RESOURCES_GET_PRIVATE (object);
-
-	free_event_sources (priv);
-
-	G_OBJECT_CLASS (tracker_resources_parent_class)->finalize (object);
-}
-
 static void
 tracker_resources_class_init (TrackerResourcesClass *klass)
 {
@@ -450,7 +440,7 @@ static void
 on_statements_committed (gpointer user_data)
 {
 	TrackerResources *resources = user_data;
-	GPtrArray *events;
+	GArray *events;
 	GHashTable *writebacks;
 	TrackerResourcesPrivate *priv;
 
@@ -467,16 +457,14 @@ on_statements_committed (gpointer user_data)
 		event_sources = priv->event_sources;
 
 		for (i = 0; i < events->len; i++) {
-			GValueArray *event = events->pdata[i];
-			const gchar *uri = g_value_get_string (g_value_array_get_nth (event, 0));
-			const gchar *predicate = g_value_get_string (g_value_array_get_nth (event, 1));
-			const gchar *rdf_class = g_value_get_string (g_value_array_get_nth (event, 2));
-			TrackerDBusEventsType type = g_value_get_int (g_value_array_get_nth (event, 3));
+			TrackerEvent *event;
+
+			event = &g_array_index (events, TrackerEvent, i);
 
 			for (l = event_sources; l; l = l->next) {
 				TrackerResourceClass *class_ = l->data;
-				if (g_strcmp0 (rdf_class, tracker_resource_class_get_rdf_class (class_)) == 0) {
-					tracker_resource_class_add_event (class_, uri, predicate, type);
+				if (g_strcmp0 (tracker_class_get_uri (event->class), tracker_resource_class_get_rdf_class (class_)) == 0) {
+					tracker_resource_class_add_event (class_, event->subject, tracker_property_get_uri (event->predicate), event->type);
 					if (!to_emit) {
 						to_emit = g_hash_table_new (NULL, NULL);
 					}
@@ -575,6 +563,23 @@ tracker_resources_prepare (TrackerResources *object,
 	priv->event_sources = event_sources;
 }
 
+static void
+tracker_resources_finalize (GObject      *object)
+{
+	TrackerResourcesPrivate *priv;
+
+	priv = TRACKER_RESOURCES_GET_PRIVATE (object);
+
+	tracker_data_remove_insert_statement_callback (on_statement_inserted, object);
+	tracker_data_remove_delete_statement_callback (on_statement_deleted, object);
+	tracker_data_remove_commit_statement_callback (on_statements_committed, object);
+	tracker_data_remove_rollback_statement_callback (on_statements_rolled_back, object);
+
+	free_event_sources (priv);
+
+	G_OBJECT_CLASS (tracker_resources_parent_class)->finalize (object);
+}
+
 void
 tracker_resources_unreg_batches (TrackerResources *object,
                                  const gchar      *old_owner)
